#!/bin/sh

# SEE COMMENTS AT END.
# Avery Earle, ae@play.psych.mun.ca, July 11, 1997.

AWK=/usr/local/bin/gawk

$AWK '

BEGIN {

# Need a list of usernames and home directories.
# Later on we will need
# to decide whether to create a file ~user/.qmail or
# a file /var/qmail/alias/.qmail-user
#
# Remember that the existance of a username takes
# precedence over a /var/qmail/alias/.qmail-* file.

	FS=":"
	while (getline <"/etc/passwd" > 0) {
		HomeDir[$1] = $6
	}
	FS=" "

	alias = ""
}

# a comment
$0 ~ /^#/ {
	dump()
	next
}

# a blank line / white-space-only line
NF==0 {
	dump()
	next
}

# leading white space -- a continuation line
$0 ~ /^[ \t]/ {
	alias = alias " " $0
	next
}

# what is left must be text flush with the left margin --
# this is a new alias.

$0 ~ /^[a-zA-Z]/ {
	dump()
	alias = $0
	next
}

END {
	dump()
}

function dump()
{
	if (length(alias)==0) return;

	# there ought to be a colon

	j = split(alias, part, ":")
	if (j==0) return

	# trim leading, trailing whitespace,
	# reduce all whitespace to a single space.
	for (j in part) {
		gsub(/^[ \t]+/, "", part[j])
		gsub(/[ \t]$+/, "", part[j])
		gsub(/[ \t]+/, " ", part[j])
	}

	if (part[1] in HomeDir) {
		if (HomeDir[part[1]] == "/") {
			# skip accounts for which home dir is /
			alias = ""
			return
		}
		outfile = HomeDir[part[1]] "/.qmail"
		print "[ -f " outfile " ] || touch " outfile " ; chmod 644 "\
			outfile " ; chown " part[1] " " outfile
	}
	else {
		outfile = "/var/qmail/alias/.qmail-" part[1]
		print "[ -f " outfile " ] || touch " outfile " ; chmod 644 "\
			outfile
	}

	# does this look like program delivery ?
	if ( (part[2] ~ /^"[ ]*[|]/) && part[2] ~ /"$/ ) {
		sub(/^"[ ]*[|]/, "| preline ", part[2])
		sub(/"$/, "", part[2])
		print "echo \"" part[2] "\" > " outfile
	}
	else {
		split(part[2], address, ",")
		for (j in address) {
			print "echo " address[j] " >> " outfile
		}
	}
	alias = ""
}
# awk secipt ends with following apostophe
'

exit

# ------------------------------------------------------------
# COMMENTS.
# ------------------------------------------------------------

This is a rough cut of a script for converting lines of /etc/aliases
to instructions for generating corresponding .qmail files. Typical
invocation is

        cat /etc/aliases | ./this-script > temp.sh

This thing seems to work, but users are urged to INSPECT THE OUTPUT
BEFORE EXECUTING IT.

# ------------------------------------------------------------

Further, qmail users will remember that a username will take precedence
over a file in ~alias/; thus the following is WRONG (that is, useless)

	file:		 /var/qmail/alias/.qmail-petidomo
	content:	| preline /u6/petidomo/bin/listserv

	file:		/var/qmail/alias/.qmail-petidomo-manager
	content:	fred

and what is needed is

	file:		~petidomo/.qmail
	content:	| preline /u6/petidomo/bin/listserv

	file:		~petidomo/.qmail-manager
	content:	fred


# ------------------------------------------------------------

Qmail's companion package qmsmac is NOT FOR PROGRAM DELIVERY
and so does not help with petidomo.

# ------------------------------------------------------------

(Further gotcha reports welcome.)



