#! /bin/sh

echo -n "Determining version... "
vers=`sed -n -e '/^#define[	 ][	 ]*VERS/{
		s/^.*[	 ]"\(.*\)"$/\1/
		p
		s/.*[^0-9]\([0-9][0-9]*\)$/\1/
		p
	}' <src/petidomo/version.h` || (echo "*** ERROR: Can't parse version.h" ; exit 1)

package=`echo $vers | sed -e 's/^\([^ ]*\).*$/\1/'`
version=`echo $vers | sed -e 's/^[^ ]* \([^ ]*\).*$/\1/'`
bump=`echo $vers | sed -e 's/^[^ ]* [^ ]* \([^ ]*\).*$/\1/'`
newbump=$(($bump + 1))
newversion=${version%$bump}${newbump}
echo ${package} ${version}
tagversion=`echo $version | sed -e 's/\./-/g'`

echo "Tagging ${package}-${tagversion} label to all modules."
cvs -q tag ${package}-${tagversion}

echo -n "Bumping version number in version.h... "
sed -e "/^#define[	 ][	 ]*VERS/s/${version}/${newversion}/" <src/petidomo/version.h >src/petidomo/version.h.new
mv src/petidomo/version.h.new src/petidomo/version.h
echo done

echo "Committing new version.h file."
cvs -q commit -m "Bumped ${package} version to ${newversion}" src/petidomo/version.h

echo "Finished."
