#! /bin/sh
#
# Install Petidomo on a machine which does already have a user and a
# group of name "petidomo".
#
# $Id: install-from-bindist.sh,v 1.5 1998/08/20 16:15:41 simons Exp $
#

#
# Determine a few paths.
#
CLEAR=`( [ -x /bin/clear ] && echo "/bin/clear") || \
	( [ -x /usr/bin/clear ] && echo "/usr/bin/clear") || \
	( [ -x /usr/local/bin/clear ] && echo "/usr/local/bin/clear") || \
	( [ -x /usr/bsd/clear ] && echo "/usr/bsd/clear") || \
	echo ":" `
CSH=`( [ -x /bin/csh ] && echo "/bin/csh") || \
	( [ -x /sbin/csh ] && echo "/sbin/csh") || \
	echo ":" `
SENDMAIL=`( [ -x /usr/sbin/sendmail ] && echo "/usr/sbin/sendmail") || \
	( [ -x /usr/etc/sendmail ] && echo "/usr/etc/sendmail") || \
	( [ -x /usr/lib/sendmail ] && echo "/usr/lib/sendmail") || \
	( [ -x /usr/local/sbin/sendmail ] && echo "/usr/local/sbin/sendmail") || \
	( [ -x /usr/local/etc/sendmail ] && echo "/usr/local/etc/sendmail") || \
	( [ -x /usr/local/lib/sendmail ] && echo "/usr/local/lib/sendmail") || \
	echo "sendmail" `
HOSTNAME=`( [ -x /bin/hostname ] && echo "/bin/hostname") || \
	( [ -x /sbin/hostname ] && echo "/sbin/hostname") || \
	( [ -x /usr/bin/hostname ] && echo "/usr/bin/hostname") || \
	( [ -x /usr/sbin/hostname ] && echo "/usr/sbin/hostname") || \
	( [ -x /usr/ucb/hostname ] && echo "/usr/ucb/hostname") || \
	( [ -x /usr/bsd/hostname ] && echo "/usr/bsd/hostname") || \
	echo "hostname" `
NEWALIASES=`( [ -x /bin/newaliases ] && echo "/bin/newaliases" ) || \
	( [ -x /usr/bin/newaliases ] && echo "/usr/bin/newaliases" ) || \
	( [ -x /usr/sbin/newaliases ] && echo "/usr/sbin/newaliases" ) || \
	( [ -x /usr/local/bin/newaliases ] && echo "/usr/local/bin/newaliases" ) || \
	( [ -x /usr/local/sbin/newaliases ] && echo "/usr/local/sbin/newaliases" ) || \
	echo ":" `

#
# Assert everything is prepared.
#
$CLEAR
echo "         Petidomo 2.1 installation/update procedure"
echo ""
echo "Did you prepare the machine for the installation or upgrade"
echo "process the way described in the README file?    [yes/no]"
read input

if [ ! "$input" = "yes" ]; then
    echo ""
    echo "** Please do before running the install script."
    exit 1;
fi

#
# Determine the Petidomo home directory.
#
pdhome=`$CSH -c "echo ~petidomo" | sed -e 's#/$##'`
echo
echo "Installing Petidomo in $pdhome..."

#
# Find out whether this is an update.
#
echo ""
echo 'Are you updating an already existing installation to the'
echo "Petidomo 2.1 version?                           [yes/no]"
read input

if [ "$input" = "yes" ]; then
    newinstall="no"
    echo ""
    echo "Then I will just update the binaries, without changing the"
    echo "config files."
    currdir=`pwd`
    (
      cd $pdhome
      tar -xpf $currdir/petidomo-bin.tar bin
    )
    chown -R petidomo $pdhome/bin
    chgrp -R petidomo $pdhome/bin
    chmod 751 $pdhome $pdhome/bin
    chmod 711 $pdhome/bin/*
    chmod 6111 $pdhome/bin/petidomo
    chown root $pdhome/bin/petidomoconf.cgi
    chmod 6111 $pdhome/bin/petidomoconf.cgi
else
    newinstall="yes"
    currdir=`pwd`
    (
      cd $pdhome
      tar -xpf $currdir/petidomo-bin.tar
    )
    chown -R petidomo $pdhome
    chgrp -R petidomo $pdhome
    chmod 751 $pdhome $pdhome/bin
    chmod 770 $pdhome/lists $pdhome/etc $pdhome/crash
    chmod 711 $pdhome/bin/*
    chmod 6111 $pdhome/bin/petidomo
    chmod 640 $pdhome/etc/*
    chown root $pdhome/bin/petidomoconf.cgi
    chmod 6111 $pdhome/bin/petidomoconf.cgi
fi

#
# Install the HTML config manager.
#
echo ""
echo "If you want to install the Petidomo CGI Configuration Manager"
echo "now, please enter the path to your httpd's cgi-bin directory"
echo "now. Just press return to skip this part."
read input
if [ "$input" = "" ]; then
    echo "Skipping installation of cgi manager."
else
    mv $pdhome/bin/petidomoconf.cgi $input/petidomoconf.cgi
    echo ""
    echo "The configuration manager should now be available under the"
    echo "following URL:  http://localhost/cgi-bin/petidomoconf.cgi"
fi

#
# If this is a new installation, we need to insert hostname, password
# and path to sendmail in the master config file.
#
if [ "$newinstall" = "yes" ]; then
    #
    # Read the master password.
    #
    echo ""
    echo "Now you can set the master password for Petidomo. Please enter"
    echo "the password now, but be aware that the password will be echoed"
    echo "to the screen!"
    read input
    if [ "$input" = "" ]; then
	echo 'Setting password to default "open sesame".'
	password="open sesame"
    else
	password="$input"
    fi

    #
    # Determine the hostname.
    #
    fqdn=`$HOSTNAME`
    echo ""
    echo "I have determined your host name as $fqdn."
    echo "Is this correct?      [yes/no]"
    read input
    if [ ! "$input" = "yes" ]; then
	echo ""
	echo "Please enter your fully qualified domain name:"
	read fqdn
    fi

    #
    # Patch these values into the master config file.
    #
    sed -e "s#@HOSTNAME@#$fqdn#" \
	-e "s#@ADMINPASSWORD@#$password#" \
	-e "s#@MTA@#$SENDMAIL#" \
	</$pdhome/etc/petidomo.conf >$pdhome/etc/petidomo.conf.new
    mv $pdhome/etc/petidomo.conf.new $pdhome/etc/petidomo.conf
    chown petidomo $pdhome/etc/petidomo.conf
    chgrp petidomo $pdhome/etc/petidomo.conf
    chmod 660 $pdhome/etc/petidomo.conf
fi

#
# Add the required aliases in case they're no there already.
#
have_aliases=`grep petidomo /etc/aliases`
if [ "$have_aliases" = "" ]; then
    echo ""
    echo "Petidomo requires a few aliases in your '/etc/aliases' file."
    echo "Would you like me to add them automatically now?   [yes/no]"
    read input
    if [ ! "$input" = "yes" ]; then
	echo "The required aliases are not added."
    else
	echo
	echo "Please enter the e-mail address of the person who is responsible"
	echo "for Petidomo on this machine:"
	read input
	if [ ! "$input" = "" ]; then
	    manager=$input
	else
	    echo "No address entered. Using 'root' as default."
	    manager='root'
	fi
	echo >>/etc/aliases "#"
	echo >>/etc/aliases "# Mailing List Stuff"
	echo >>/etc/aliases "#"
	echo >>/etc/aliases "petidomo-manager: $manager"
	echo >>/etc/aliases "petidomo: \"|$pdhome/bin/listserv\""
	${NEWALIASES}
    fi
    else
	echo "The required aliases are there already, skipping."
fi

#
# That's all!
#
echo ""
echo "Congratulations, the installation is complete. You may now use"
echo "the CGI manager to create a mailing list and to configure the"
echo "installation according to your wishes."
echo ""
echo "I know this hint is futile anyway, but nonetheless I'd like to"
echo "point out that reading the user manual might help getting"
echo "everything to work as expected. :-)"
echo ""
echo "If you encounter any bugs, please use the installed send-pr"
echo "script to report the problem, by calling ~petidomo/bin/send-pr."
echo ""
echo "                             Peter Simons <simons@petidomo.com>"
