/*
 *      $Source: /usr/local/libdata/cvs/simons/petidomo/src/htmlconf/aliases.c,v $
 *      $Revision: 1.6 $
 *      $Date: 1997/10/19 14:37:35 $
 *
 *      Copyright (C) 1997 by Peter Simons, Germany
 *      All rights reserved.
 */

#include <string.h>
#include <errno.h>
#include <unistd.h>

#include <text.h>
#include <cgi.h>
#include <petidomo.h>

void SetUIDRoot(void);
void SetUIDPetidomo(void);

int
runNewAliases(void)
{
    pid_t   child_pid;

    child_pid = fork();
    switch(child_pid) {
      case 0:
          /* Child */
          execl("//bin/sh", "sh", "-c", "newaliases", NULL);
          return -1;
      case -1:
          /* Error */
	  puts("<H3>ERROR</H1>");
	  puts("An attempt to fork() failed:<P>\n");
	  printf("<CENTER><CODE>%s</CODE></CENTER><P>\n", strerror(errno));
	  puts("Please contact the administrator with this problem.<P>\n");
          syslog(LOG_ERR, "Couldn't fork: %m");
          return -1;
    }
    return 0;
}



int
createMailinglistAlias(const char * listname)
{
    const struct PD_Config *  MasterConfig = getMasterConfig();
    FILE *    fh;

    SetUIDRoot();

    /* Add the aliases to "/etc/aliases". */

    fh = fopen("/etc/aliases", "a");
    if (fh == NULL) {
	SetUIDPetidomo();
	return -1;
    }

    fprintf(fh, "\n");
    fprintf(fh, "%s: \"|%s/bin/hermes %s\"\n", listname, MasterConfig->basedir, listname);
    fprintf(fh, "%s-request: \"|%s/bin/listserv %s\"\n", listname, MasterConfig->basedir,
	    listname);
    fprintf(fh, "%s-owner: petidomo-manager\n", listname);
    fclose(fh);

    /* Run 'newaliases'. */

    runNewAliases();

    /* Go back to user 'petidomo' and return. */

    SetUIDPetidomo();
    return 0;
}

int
removeMailinglistAlias(const char * listname)
{
    FILE *    fh;
    char *    aliases;
    char *    currLine;
    char *    nextLine;
    int       listname_len = strlen(listname);

    if (listname_len < 1)
      return -1;		/* assertion */

    SetUIDRoot();

    /* Load "/etc/aliases". */

    aliases = loadfile("/etc/aliases");
    if (aliases == NULL) {
	SetUIDPetidomo();
	return -1;
    }

    /* Write the file back and skip the aliases for that list. */

    fh = fopen("/etc/aliases", "w");
    if (fh == NULL) {
	SetUIDPetidomo();
	return -1;
    }
    for (currLine = aliases; *currLine != '\0'; currLine = nextLine) {
	nextLine = text_find_next_line(currLine);

	if (!strncasecmp(currLine, listname, listname_len)) {
	    if (currLine[listname_len] == ':' ||
		!strncasecmp(currLine+listname_len, "-request:", 9) ||
		!strncasecmp(currLine+listname_len, "-owner:", 7))
	      continue;
	}

	fwrite(currLine, sizeof(char), nextLine - currLine, fh);
	if (ferror(fh)) {
	    fclose(fh);
	    SetUIDPetidomo();
	    return -1;
	}
    }
    fclose(fh);
    SetUIDPetidomo();

    /* Run 'newaliases'. */

    runNewAliases();

    /* Go back to user 'petidomo' and return. */

    SetUIDPetidomo();
    return 0;
}

