/*
 *      $Source: /usr/local/libdata/cvs/simons/petidomo/src/htmlconf/genlistindex.c,v $
 *      $Revision: 1.8 $
 *      $Date: 1998/06/01 17:19:26 $
 *
 *      Copyright (C) 1996 by Peter Simons, Germany
 *      All rights reserved.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

#include <text.h>
#include <cgi.h>
#include <petidomo.h>

extern cgi_info      ci;
extern form_entry *  form_parameter;

void printMainPage(void);

const char select_html_pre[] = \
"<H1 ALIGN=CENTER>Petidomo 2.2 Mailinglist Configuration</H1>\n" \
"<FORM METHOD=\"POST\" ACTION=\"%s\">\n" \
"<INPUT TYPE=hidden NAME=page VALUE=listselect>\n" \
"<INPUT TYPE=hidden NAME=password VALUE=\"%s\">\n" \
"Please select the mailing list, you'd like to configure: <BR>\n" \
"<CENTER>\n" \
"<TABLE>\n";

const char select_html_mid[] = \
"</TABLE>\n" \
"</CENTER><P>\n" \
"<INPUT TYPE=\"submit\"><INPUT TYPE=\"reset\"><P>\n";

const char select_html_post[] = \
"</FORM>\n" \
"</BODY>\n" \
"</HTML>\n";

bool
isValidListName(const char * listname)
{
    struct stat   sb;
    char *        buffer;

    assert(listname != NULL);

    buffer = text_easy_sprintf("lists/%s", listname);
    if (stat(buffer, &sb) != 0)
      return FALSE;		/* Doesn't exist at all. */
    else if ((sb.st_mode & S_IFDIR) == 0)
      return FALSE;		/* Entry isn't a directory. */
    else {
	buffer = text_easy_sprintf("lists/%s/config", listname);
	if (stat(buffer, &sb) != 0)
	  return FALSE;
    }
    return TRUE;
}

void
genListIndex(void)
{
    DIR *            dirp;
    struct dirent *  entry;
    char *           password;
    unsigned int     entry_num;

    password = parmval(form_parameter, "password");

    /* Do sanity checks. */

    if (password == NULL) {
	printMainPage();
	return;
    }

    /* Print Preamble. */

    printf(select_html_pre, ci.script_name, password);

    /* Generate a list of mailing lists, that may be accessed with
       that password. */

    entry_num = 0;
    dirp = opendir("lists");
    if (dirp == NULL) {
	printf("An internal error has occured while processing your request. The\n" \
	       "server administrator has been notified. You don't need to re-submit\n" \
	       "your request, it will be processed as soon as the problem has been\n" \
	       "remedied.\n");
	syslog(LOG_ERR, "Failed to read directory \"lists\": %m");
	exit(0);
    }
    while((entry = readdir(dirp)) != NULL) {
	if (!strcasecmp(entry->d_name, ".") || !strcasecmp(entry->d_name, ".."))
	  continue;
	if (isValidListName(entry->d_name) == FALSE)
	  continue;
	if (isValidAdminPassword(password, entry->d_name) == FALSE)
	  continue;

	entry_num++;

	/* Print stuff to the page. */

	printf("  <TR><TD><INPUT TYPE=radio NAME=listname VALUE=\"%s\"> %s </TR>\n",
	       entry->d_name, entry->d_name);
    }
    closedir(dirp);

    /* Print Midfix. */

    puts(select_html_mid);

    if (entry_num == 0) {
	puts("You password does not authorize you to change the configuration");
	puts("of a mailing list.");
    }
    else
      printf("Found %d mailing lists.\n", entry_num);

    /* Print postfix. */

    puts(select_html_post);
}
